@@ -40,15 +40,15 @@ dtmlExampleCode = '<!-- @deprecated -->'
 pageTemplateExampleCode = \
   '<!DOCTYPE html>\n' + \
   '<html lang="en">\n' + \
-  '<tal:block tal:content="structure python:here.zmi_html_head(here,request)">zmi_html_head</tal:block>\n' + \
+  '<head tal:replace="structure python:here.zmi_html_head(here,request)">zmi_html_head</head>\n' + \
   '<body class="zmi">\n' + \
-  '<tal:block tal:content="structure python:here.zmi_body_header(here,request)">zmi_body_header</tal:block>\n' + \
+  '<header tal:replace="structure python:here.zmi_body_header(here,request)">zmi_body_header</header>\n' + \
   '<div id="zmi-tab">\n' + \
-  '<tal:block tal:content="structure python:here.zmi_breadcrumbs(here,request)">zmi_breadcrumbs</tal:block>\n' + \
-  '</div><!-- #zmi-tab -->\n' + \
+  '<tal:block tal:replace="structure python:here.zmi_breadcrumbs(here,request)">zmi_breadcrumbs</tal:block >\n' + \
   '<script>\n' + \
   '</script>\n' + \
-  '<tal:block tal:content="structure python:here.zmi_body_footer(here,request)">zmi_body_footer</tal:block>\n' + \
+  '<div>\n' + \
+  '<footer tal:replace="structure python:here.zmi_body_footer(here,request)">zmi_body_footer</footer>\n' + \
   '</body>\n' + \
   '</html>\n'
 
@@ -181,6 +181,22 @@ class ZMSMetacmdProvider(
         newNodes)
 
 
+    """
+    @see IRepositoryProvider
+    """
+    def translateRepositoryModel(self, r):
+      l = []
+      for k in r:
+          v  = r[k]
+          # map implementation
+          impl = v['Impl'][0]
+          v['meta_type'] = impl['type']
+          v['data'] = impl['data']
+          del v['Impl']
+          l.append(v)
+      return l
+
+
     ############################################################################
     #
     #  XML IM/EXPORT
@@ -190,9 +206,8 @@ class ZMSMetacmdProvider(
     # ------------------------------------------------------------------------------
     #  ZMSMetacmdProvider.importXml
     # ------------------------------------------------------------------------------
-    def _importXml(self, item, createIfNotExists=1):
-      id = item['id']
-      if createIfNotExists == 1:
+    def _importXml(self, item):
+        id = item['id']
         
         # Delete existing object.
         try: self.delMetacmd(id)
@@ -218,13 +233,13 @@ class ZMSMetacmdProvider(
           newData, newExecution, newDescription, newIconClazz, newMetaTypes, newRoles, \
           newNodes)
 
-    def importXml(self, xml, createIfNotExists=1):
+    def importXml(self, xml):
       v = standard.parseXmlString(xml)
       if isinstance(v, list):
         for item in v:
-          id = self._importXml(item, createIfNotExists)
+          id = self._importXml(item)
       else:
-        id = self._importXml(v, createIfNotExists)
+        id = self._importXml(v)
 
 
     # ------------------------------------------------------------------------------
@@ -371,9 +386,8 @@ class ZMSMetacmdProvider(
       obs = self.commands
       if sort:
         obs = [self.getMetaCmd(x['id']) for x in obs]
-        obs = [(x['name'], x) for x in obs if x is not None]
-        obs = sorted(obs)
-        obs = [x[1] for x in obs]
+        obs = [x for x in obs if x]
+        obs = sorted(obs,key=lambda x: x['name'])
       ids = [x['id'] for x in obs]
       return ids
 
@@ -548,7 +562,7 @@ class ZMSMetacmdProvider(
             self.importXml(xml=f)
           else:
             filename = REQUEST['init']
-            self.importConf(filename, createIfNotExists=1)
+            self.importConf(filename)
           message = self.getZMILangStr('MSG_IMPORTED')%('<i>%s</i>'%f.filename)
         
         # Insert.
