@@ -365,11 +365,7 @@ class ZMSRepositoryManager(
                 f = open(filepath,"wb")
                 if isinstance(data,str):
                   data = data.encode("utf-8")
-                try:
-                  f.write(data)
-                except:
-                  # f.write('## ERROR ON FILE WRITE')
-                  f.write(data.encode("utf-8"))
+                f.write(data)
                 f.close()
               else:
                 failure.append('%s is None'%file)
