@@ -155,7 +155,7 @@ class DeprecatedAPI(object):
       try:
         s = stylesheet(self)
       except:
-        s = bytes(stylesheet).decode()
+        s = standard.pystr(stylesheet)
       css.append("/* ######################################################################")
       css.append("   ### %s"%stylesheet.absolute_url())
       css.append("   ###################################################################### */")
@@ -380,7 +380,7 @@ class DeprecatedAPI(object):
     warn(self, 'aggregate_list', 'Products.zms.standard.aggregate_list')
     return standard.aggregate_list(l, i)
 
-  def url_append_params(self, url, dict, sep='&amp;'):
+  def url_append_params(self, url, dict, sep='&'):
     warn(self, 'url_append_params', 'Products.zms.standard.url_append_params')
     return standard.url_append_params(url, dict, sep)
 
