@@ -20,7 +20,6 @@
 from App.Common import package_home
 import OFS.misc_
 import copy
-import six
 from zope.interface import implementer
 # Product Imports.
 from Products.zms import IZMSLocale
@@ -32,8 +31,7 @@ from Products.zms import standard
 # ------------------------------------------------------------------------------
 #  importXml
 # ------------------------------------------------------------------------------
-def _importXml(self, item, createIfNotExists=1):
-  if createIfNotExists:
+def _importXml(self, item):
     key = item['key']
     lang_dict = self.get_lang_dict()
     lang_dict[key] = {}
@@ -42,7 +40,7 @@ def _importXml(self, item, createIfNotExists=1):
         lang_dict[key][langId] = item[langId]
     self.setConfProperty('ZMS.custom.langs.dict', lang_dict.copy())
 
-def importXml(self, xml, createIfNotExists=1):
+def importXml(self, xml):
   if not isinstance(xml, str):
     xml = xml.read()
   value = standard.parseXmlString(xml)
@@ -72,9 +70,9 @@ def importXml(self, xml, createIfNotExists=1):
           r += 1
   if isinstance(value, list):
     for item in value:
-      _importXml( self, item, createIfNotExists)
+      _importXml( self, item)
   else:
-    _importXml( self, value, createIfNotExists)
+    _importXml( self, value)
 
 def exportXml(self, ids, REQUEST=None, RESPONSE=None):
   value = []
@@ -297,10 +295,7 @@ class MultiLanguageManager(object):
       """
       Returns language-string for current content-language.
       """
-      lang_str = self._getLangStr(key,lang)
-      if six.PY2:
-        lang_str = standard.pybytes(lang_str,'utf-8')
-      return lang_str
+      return self._getLangStr(key,lang)
 
 
     # --------------------------------------------------------------------------
@@ -662,7 +657,7 @@ class MultiLanguageManager(object):
             importXml(self, xml=f)
           else:
             filename = REQUEST['init']
-            self.importConf(filename, createIfNotExists=1)
+            self.importConf(filename)
           message = self.getZMILangStr('MSG_IMPORTED')%('<i>%s</i>'%filename)
         
         # Return with message.
