@@ -316,7 +316,7 @@ class ObjAttrs(object):
         filteredMetaObjAttrs = [x for x in metaObj['attrs'] if x['id'] == 'format']
         if len(filteredMetaObjAttrs) == 1:
           if REQUEST.get('ZMS_INSERT'):
-            default = standard.dt_exec(self, standard.pystr( filteredMetaObjAttrs[0].get('default', '')))
+            default = standard.dt_exec(self, str( filteredMetaObjAttrs[0].get('default', '')))
             if default:
               fmt = default
             else:
@@ -330,7 +330,7 @@ class ObjAttrs(object):
           REQUEST.set('data', data)
           text_fmt = self.getTextFormat(fmt, REQUEST)
           form_fixed = form_fixed or ( text_fmt is not None and not text_fmt.getTag() and not text_fmt.getSubTag())
-        ltxt = standard.pystr(value).lower()
+        ltxt = str(value).lower()
         form_fixed = form_fixed or ( ltxt.find( '<form') >= 0 or ltxt.find( '<input') >= 0 or ltxt.find( '<script') >= 0)
         if form_fixed:
           css = 'form-control form-fixed'
@@ -341,11 +341,11 @@ class ObjAttrs(object):
       
       #-- Image-Fields.
       elif inputtype == 'image':
-        return self.f_selectImage(self, ob=self, fmName=fmName, elName=elName, value=value, key=obj_attr['id'], metaObj=metaObj, lang=lang, REQUEST=REQUEST)
+        return self.f_selectImage(self, ob=self, fmName=fmName, elName=elName, value=_blobfields.MyBlobDelegate(value), key=obj_attr['id'], metaObj=metaObj, lang=lang, REQUEST=REQUEST)
       
       #-- File-Fields.
       elif inputtype == 'file':
-        return self.f_selectFile(self, ob=self, fmName=fmName, elName=elName, value=value, key=obj_attr['id'], metaObj=metaObj, lang=lang, REQUEST=REQUEST)
+        return self.f_selectFile(self, ob=self, fmName=fmName, elName=elName, value=_blobfields.MyBlobDelegate(value), key=obj_attr['id'], metaObj=metaObj, lang=lang, REQUEST=REQUEST)
       
       #-- Password-Fields.
       if inputtype == 'password':
@@ -1142,7 +1142,11 @@ class ObjAttrs(object):
           rtn['height'] = int(h)
         rtn['content_type'] = file.content_type
         rtn['filename'] = file.title
-        rtn['src'] = self.url_append_params(file.absolute_url(), {'ts':time.time()})
+        fileurl = file.absolute_url()
+        if len(fileurl.split('//')) > 2:
+        # Fix ocasionally missing temp_folder path
+          fileurl = '%s//temp_folder/%s'%(fileurl.rsplit('//',1)[0],fileurl.rsplit('//',1)[1])
+        rtn['src'] = self.url_append_params(fileurl, {'ts':time.time()})
       # Return JSON.
       if format == 'json':
         rtn = self.str_json(rtn)
