@@ -81,7 +81,7 @@ def getText(nodelist, encoding='utf-8'):
     for childNode in node.childNodes:
       if childNode.nodeType == childNode.TEXT_NODE:
         rc.append(childNode.data)
-  return standard.pystr(''.join(rc).encode(encoding))
+  return ''.join(rc)
 
 # ------------------------------------------------------------------------------
 #  _xmllib.parseString:
@@ -312,7 +312,7 @@ def xmlOnUnknownEndTag(self, sTagName):
         try:
           data = standard.hex2bin(cdata)
         except:
-          data = standard.pybytes(cdata,'utf-8')
+          data = bytes(cdata,'utf-8')
         value['data'] = data
       self.dValueStack.append(value)
 
@@ -509,8 +509,8 @@ def toCdata(self, s, xhtml=False):
 
   # Return Text in CDATA.
   elif s is not None:
-    if standard.is_bytes(s):
-      s = standard.pystr(s)
+    if isinstance(s,bytes):
+      s = str(s)
     # Hack for invalid characters
     s = s.replace(chr(30), '')
     # Hack for nested CDATA
@@ -548,8 +548,11 @@ def toXml(self, value, indentlevel=0, xhtml=False, encoding='utf-8'):
       xml.append(' type="file"')
       xml.append('>')
       data = zopeutil.readData(value)
-      if content_type.startswith('text/') or content_type in ['application/css','application/javascript','image/svg']:
-        data = standard.pystr(data,'utf-8')
+      if [x for x in ['text/','application/css','application/javascript','image/svg'] if content_type.startswith(x)]:
+        if type(data) is bytes:
+          data = data.decode('utf-8')
+        else:
+          data = str(data,'utf-8')
       cdata = None
       # Ensure CDATA is valid.
       try:
@@ -558,7 +561,7 @@ def toXml(self, value, indentlevel=0, xhtml=False, encoding='utf-8'):
         rv = p.Parse('<?xml version="1.0" encoding="utf-8"?><%s>%s</%s>'%(tagname,cdata,tagname), 1)
       # Otherwise use binary encoding.
       except:
-        cdata = standard.bin2hex(standard.pybytes(data))
+        cdata = standard.bin2hex(bytes(data))
       xml.append(cdata)
       xml.append('</%s>' % tagname)
 
@@ -624,7 +627,7 @@ def toXml(self, value, indentlevel=0, xhtml=False, encoding='utf-8'):
         xml.append(toCdata(self, value, xhtml))
 
   # Return xml.
-  return ''.join([standard.pystr(x) for x in xml])
+  return ''.join([str(x) for x in xml])
 
 
 # ------------------------------------------------------------------------------
@@ -661,15 +664,11 @@ def getAttrToXml(self, base_path, data2hex, obj_attr, REQUEST):
 
     #-- Text-Fields
     elif datatype in _globals.DT_TEXTS:
-      request = self.REQUEST
-      request.set('ExtensionPoint.ZReferableItem.getRefObjPath','disabled')
       value = self.validateInlineLinkObj(value)
       xml += toXml(self, value)
       
     #-- Url-Fields
     elif datatype == _globals.DT_URL:
-      request = self.REQUEST
-      request.set('ExtensionPoint.ZReferableItem.getRefObjPath','disabled')
       value = self.validateLinkObj(value)
       xml += toXml(self, value)
     
@@ -815,7 +814,7 @@ class XmlAttrBuilder(object):
       p.EndNamespaceDeclHandler = self.OnEndNamespaceDecl
 
       #### parsing ####
-      if standard.is_bytes(input):
+      if isinstance(input,bytes):
         # input is a string!
         rv = p.Parse(input, 1)
       else:
@@ -897,7 +896,7 @@ class XmlAttrBuilder(object):
         try:
           data = standard.hex2bin(cdata)
         except:
-          data = standard.pybytes(cdata,'utf-8')
+          data = bytes(cdata,'utf-8')
         file = {'data':data, 'filename':filename, 'content_type':content_type}
         objtype = attrs.get('type')
         item = _blobfields.createBlobField(None, objtype, file)
