@@ -53,8 +53,6 @@ def getInternalLinkDict(self, url):
   d = {}
   # Params.
   ref_params = ''
-  if url.startswith('{$__') and url.endswith('__}'):
-    url = '{$%s}'%url[len('{$__'):-len('__}')]
   if url.find(';') > 0:
     ref_params = url[url.find(';'):-1]
     url = '{$%s}'%url[2:url.find(';')]
@@ -96,7 +94,7 @@ def getInlineRefs(text):
   l = []
   p = '<a(.*?)>(.*?)<\\/a>'
   r = re.compile(p)
-  for f in r.findall(standard.pystr(text)):
+  for f in r.findall(str(text)):
     d = dict(re.findall('\\s(.*?)="(.*?)"', f[0]))
     if 'data-id' in d:
       l.append(d['data-id'])
@@ -262,13 +260,13 @@ class ZReferableItem(object):
             v = getattr(obj_vers, '%s%s'%(key, lang_suffix), None)
             if v is not None:
               if datatype in ['richtext', 'string', 'text']:
-                for iv in getInlineRefs(v):
-                  ref_ob = self.getLinkObj(str(iv))
+                for iv in getInlineRefs(str(v)):
+                  ref_ob = self.getLinkObj(iv)
                   if ref_ob is not None:
                     ref = self.getRefObjPath(ref_ob)
                     d[ref] = 1
               elif datatype in ['url']:
-                ref_ob = self.getLinkObj(str(v))
+                ref_ob = self.getLinkObj(v)
                 if ref_ob is not None:
                   ref = self.getRefObjPath(ref_ob)
                   d[ref] = 1
@@ -328,12 +326,11 @@ class ZReferableItem(object):
   # ----------------------------------------------------------------------------
   def validateInlineLinkObj(self, text):
     if not int(self.getConfProperty('ZReferableItem.validateLinkObj', 1)): return text
-    text = standard.pybytes(text)
     for pq in [('<a(.*?)>', 'href'), ('<img(.*?)>', 'src')]:
       p = pq[0]
       q = pq[1]
       r = re.compile(p)
-      for f in r.findall(text):
+      for f in r.findall(str(text)):
         d = dict(re.findall('\\s(.*?)="(.*?)"', f))
         if 'data-id' in d:
           old = p.replace('(.*?)', f)
@@ -341,8 +338,9 @@ class ZReferableItem(object):
           ild = getInternalLinkDict(self, url)
           for k in ild:
             d[{'data-url':q}.get(k, k)] = ild[k]
-          new = p.replace('(.*?)', ' '.join(['']+['%s="%s"'%(x,standard.pybytes(d[x])) for x in d]))
+          new = p.replace('(.*?)', ' '.join(['']+['%s="%s"'%(x,d[x]) for x in d]))
           if old != new:
+            # @FIXME UnicodeDecodeError: 'ascii' codec can't decode byte 0x## in position ###: ordinal not in range(128)
             text = text.replace(old, new)
     return text
 
@@ -367,7 +365,7 @@ class ZReferableItem(object):
   #  Resolves internal/external links and returns Object.
   # ----------------------------------------------------------------------------
   def getLinkObj(self, url, REQUEST=None):
-    ob = None 
+    ob = None
     if isInternalLink(url):
       def default(*args, **kwargs):
         self = args[0]
@@ -444,7 +442,7 @@ class ZReferableItem(object):
   # ----------------------------------------------------------------------------
   def tal_anchor(self, href, target='', attrs={}, content=''):
     filtered_attrs_keys = [x for x in attrs if x]
-    str_attrs = ' '.join(['%s=\042%s\042'%(standard.pystr(x),str(attrs[x])) for x in filtered_attrs_keys])
+    str_attrs = ' '.join(['%s=\042%s\042'%(str(x),str(attrs[x])) for x in filtered_attrs_keys])
     return '<a href="%s" %s %s>%s</a>'%(href, ['', ' target="%s"'%target][int(len(target)>0)], str_attrs, content)
 
 ################################################################################
