@@ -28,7 +28,6 @@ Scripts.  It can be accessed from Python with the statement
 # Imports.
 from __future__ import absolute_import
 from AccessControl.SecurityInfo import ModuleSecurityInfo
-from AccessControl import AuthEncoding
 from App.Common import package_home
 from App.config import getConfiguration
 from DateTime.DateTime import DateTime
@@ -38,7 +37,9 @@ import base64
 import cgi
 import copy
 import fnmatch
+import hashlib
 import inspect
+import io
 import json
 import logging
 import operator
@@ -54,19 +55,6 @@ from six.moves.urllib import parse as urllib_parse
 from six.moves.urllib.parse import quote as urllib_quote
 from six.moves.urllib.parse import quote_plus as urllib_quote_plus
 from six.moves.urllib.parse import unquote as urllib_unquote
-from six.moves.urllib.parse import urlparse as urllib_urlparse
-from six import BytesIO as PyBytesIO
-
-# if six.PY3:
-#   import urllib.parse                 as urllib_parse
-#   from urllib.parse import quote_plus as urllib_quote_plus
-#   from urllib.parse import unquote    as urllib_unquote
-#   from urllib.parse import urlparse   as urllib_urlparse
-# else:
-#   import urlparse                     as urllib_parse
-#   from urllib import quote_plus       as urllib_quote_plus
-#   from urllib import unquote          as urllib_unquote
-#   from urlparse import urlparse       as urllib_urlparse
 
 # Product Imports.
 from Products.zms import _globals
@@ -77,75 +65,38 @@ security = ModuleSecurityInfo('Products.zms.standard')
 
 security.declarePublic('is_str')
 security.declarePublic('is_bytes')
+def is_str(v):
+  return isinstance(v,str)
+def is_bytes(v):
+  return isinstance(v,bytes)
+
 security.declarePublic('pystr')
-security.declarePublic('pybytes')
-if six.PY2:
-  def is_str(v):
-    return isinstance(v,unicode)
-  def is_bytes(v):
-    return isinstance(v,str) or isinstance(v,bytes)
-  def pystr(object, encoding='utf-8', errors='strict'):
-    if not is_str(object):
-      if isinstance(object,str):
-        object = unicode(object,encoding,errors)
-      else:
-        object = unicode(object)
-    return object
-  def pybytes(object, encoding='utf-8', errors='strict'):
-    if is_str(object):
-      object = object.encode(encoding,errors)
-    elif not is_bytes(object):
-      object = bytes(object)
-    return object
-  def pyopen(name, mode, buffering=-1, encoding=None):
-    return open(name, mode, buffering)
-  from cgi import escape as html_escape
-if six.PY3:
-  def is_str(v):
-    return isinstance(v,str)
-  def is_bytes(v):
-    return isinstance(v,bytes)
-  pystr = str
-  pybytes = bytes
-  pyopen = open
-  from html import escape as html_escape
+pystr_ = str
+def pystr(v, encoding='utf-8', errors='ignore'):
+  if isinstance(v,bytes):
+    v = v.decode(encoding)
+  elif not isinstance(v,str):
+    try:
+      v = str(v,encoding)
+    except:
+      v = str(v)
+  return v
 
-# added in six-1.12.0
-def six_ensure_binary(s, encoding='utf-8', errors='strict'):
-    """Coerce **s** to six.binary_type.
-    For Python 2:
-      - `unicode` -> encoded to `str`
-      - `str` -> `str`
-    For Python 3:
-      - `str` -> encoded to `bytes`
-      - `bytes` -> `bytes`
-    """
-    if isinstance(s, six.text_type):
-        return s.encode(encoding, errors)
-    elif isinstance(s, six.binary_type):
-        return s
-    else:
-        raise TypeError("not expecting type '%s'" % type(s))
 
-# added in six-1.12.0
-def six_ensure_str(s, encoding='utf-8', errors='strict'):
-    """Coerce *s* to `str`.
-    For Python 2:
-      - `unicode` -> encoded to `str`
-      - `str` -> `str`
-    For Python 3:
-      - `str` -> `str`
-      - `bytes` -> decoded to `str`
-    """
-    if not isinstance(s, (six.text_type, six.binary_type)):
-        raise TypeError("not expecting type '%s'" % type(s))
-    if six.PY2 and isinstance(s, six.text_type):
-        s = s.encode(encoding, errors)
-    elif six.PY3 and isinstance(s, six.binary_type):
-        s = s.decode(encoding, errors)
-    return s
+security.declarePublic('addZMSCustom')
+def addZMSCustom(self, meta_id=None, values={}, REQUEST=None):
+  """
+  Public alias for manage_addZMSCustom
+  @param meta_id
+  @param values
+  @param REQUEST
+  @rtype: C{ZMSCustom}
+  """
+  return self.manage_addZMSCustom(meta_id, values, REQUEST)
 
 
+from html import escape as html_escape
+
 def url_quote(s):
   return urllib_quote(s)
 
@@ -206,16 +157,14 @@ def getINSTANCE_HOME():
 
 security.declarePublic('zmi_paths')
 def zmi_paths(context):
-	kw = {}
-	try:
-		from zmi.styles.subscriber import css_paths, js_paths
-		# remove zmi base css/js
-		kw["css_paths"] = css_paths(context)[:-1]
-		kw["js_paths"] = js_paths(context)[:-2]
-	except:
-		kw["css_paths"] = ("/++resource++zmi/bootstrap-4.1.1/bootstrap.min.css","/++resource++zmi/fontawesome-free-5.8.1/css/all.css",)
-		kw["js_paths"] = ("/++resource++zmi/jquery-3.2.1.min.js","/++resource++zmi/bootstrap-4.1.1/bootstrap.bundle.min.js",)
-	return kw
+  kw = {}
+  from zmi.styles.subscriber import css_paths, js_paths
+  # ZMI resources without Zope base css/js
+  # css_paths = ("/++resource++zmi/bootstrap-4.6.0/bootstrap.min.css","/++resource++zmi/fontawesome-free-5.15.2/css/all.css")
+  # js_paths = ("/++resource++zmi/jquery-3.5.1.min.js","/++resource++zmi/bootstrap-4.6.0/bootstrap.bundle.min.js",)
+  kw["css_paths"] = css_paths(context)[:-1]
+  kw["js_paths"] = js_paths(context)[:-2]
+  return kw
 
 
 security.declarePublic('FileFromData')
@@ -260,6 +209,53 @@ def set_response_headers(fn, mt='application/octet-stream', size=None, request=N
   RESPONSE.setHeader('Accept-Ranges', 'bytes')
 
 
+security.declarePublic('get_installed_packages')
+def get_installed_packages():
+  import subprocess
+  pipfreeze = subprocess.Popen("../../../bin/pip freeze --all",
+                               stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                               shell=True, cwd=getPACKAGE_HOME(), universal_newlines=True)
+  packages = pipfreeze.communicate()[0].strip()
+  return packages
+
+
+security.declarePublic('set_response_headers_cache')
+def set_response_headers_cache(context, request=None, cache_max_age=24*3600):
+  """
+  Set default and dynamic cache response headers according to ZMS_CACHE_EXPIRE_DATETIME
+  which is determined in ObjAttrs.isActive for each page element as the earliest time for invalidation.
+  @see: http://nginx.org/en/docs/http/ngx_http_headers_module.html#expires
+  @see: https://www.nginx.com/resources/wiki/start/topics/examples/x-accel/
+  @return: Tuple of expires date time in GMT as ISO8601 string and the seconds until expiration
+  """
+  if request is not None:
+    is_preview = request.get('preview', '') == 'preview'
+    is_restricted = len([ob for ob in context.breadcrumbs_obj_path(portalMaster=False)
+                         if ob.attr('attr_dc_accessrights_restricted') in [1, True]]) > 0
+
+    if is_restricted or is_preview:
+      request.RESPONSE.setHeader('Cache-Control', 'no-cache')
+      request.RESPONSE.setHeader('Expires', '-1')
+      request.RESPONSE.setHeader('Pragma', 'no-cache')
+    else:
+      request.RESPONSE.setHeader('Cache-Control',
+                                 'max-age={}, public, must-revalidate, proxy-revalidate'.format(cache_max_age))
+      now = time.time()
+      expire_datetime = DateTime(request.get('ZMS_CACHE_EXPIRE_DATETIME', now + cache_max_age))
+      t1 = expire_datetime.millis()
+      t0 = DateTime(now).millis()
+      expire_in_secs = int((t1-t0)/1000)
+      expire_datetime_gmt = expire_datetime.toZone('GMT')
+
+      if t1 > t0 and cache_max_age > expire_in_secs:
+        request.RESPONSE.setHeader('Expires', expire_datetime_gmt.asdatetime().strftime('%a, %d %b %Y %H:%M:%S %Z'))
+        request.RESPONSE.setHeader('X-Accel-Expires', expire_in_secs)
+
+      return expire_datetime_gmt.ISO8601(), expire_in_secs
+
+  return None
+
+
 security.declarePublic('umlaut_quote')
 def umlaut_quote(s, mapping={}):
   """
@@ -271,17 +267,17 @@ def umlaut_quote(s, mapping={}):
   @return: Quoted string
   @rtype: C{str}
   """
-  if is_bytes(s):
-    s = pystr(s)
+  if not isinstance(s,str):
+    s = str(s)
   for x in _globals.umlaut_map:
     mapping[x] = _globals.umlaut_map[x]
   for key in mapping:
-    s = s.replace(key, pystr(mapping[key]))
+    s = s.replace(key, str(mapping[key]))
   return s
 
 
 security.declarePublic('url_append_params')
-def url_append_params(url, dict, sep='&amp;'):
+def url_append_params(url, dict, sep='&'):
   """
   Append params from dict to given url.
   @param url: Url
@@ -305,11 +301,11 @@ def url_append_params(url, dict, sep='&amp;'):
   i = url.find(qs)
   if i >= 0:
     qs = sep
-  for key in dict.keys():
+  for key in dict:
     value = dict[key]
     if isinstance(value, list):
       for item in value:
-        qi = key + ':list=' + url_quote(pystr(item))
+        qi = key + ':list=' + url_quote(str(item))
         url += qs + qi
         qs = sep
     else:
@@ -381,9 +377,9 @@ def string_maxlen(s, maxlen=20, etc='...', encoding=None):
   @rtype: C{str}
   """
   if encoding is not None:
-    s = pystr( s, encoding)
+    s = str( s, encoding)
   else:
-    s = pystr(s)
+    s = str(s)
   # remove all tags.
   s = re.sub( '<!--(.*?)-->', '', s)
   s = re.sub( '<script((.|\n|\r|\t)*?)>((.|\n|\r|\t)*?)</script>', '', s)
@@ -435,7 +431,8 @@ def guess_content_type(filename, data):
 html_quote:
 """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
 def html_quote(v, name='(Unknown name)', md={}):
-  v = pybytes(v)
+  if not isinstance(v,str):
+    v = str(v)
   return html_escape(v, 1)
 
 
@@ -448,7 +445,7 @@ def bin2hex(m):
   @rtype: C{bytes}
   """
   import binascii
-  return six_ensure_str(binascii.hexlify(m))
+  return six.ensure_str(binascii.hexlify(m))
 
 
 def hex2bin(m):
@@ -470,10 +467,7 @@ def encrypt_schemes():
   @return: list of encryption-scheme ids
   @rtype: C{list}
   """
-  ids = []
-  for id, prefix, scheme in AuthEncoding._schemes:
-    ids.append( id)
-  return ids
+  return list(hashlib.algorithms_available)
 
 
 security.declarePublic('encrypt_password')
@@ -482,28 +476,25 @@ def encrypt_password(pw, algorithm='md5', hex=False):
   Encrypts given password.
   @param pw: Password
   @type pw: C{str}
-  @param algorithm: Encryption-algorithm (md5, sha-1, etc.)
+  @param algorithm: Encryption-algorithm (md5, sha1, etc.)
   @type algorithm: C{str}
   @param hex: Hexlify
   @type hex: C{bool}
   @return: Encrypted password
   @rtype: C{str}
   """
+  algorithm = algorithm.lower()
+  algorithm = algorithm in ['sha-1','sha'] and 'sha1' or algorithm
   enc = None
-  if algorithm.upper() == 'SHA-1':
-    import sha
-    enc = sha.new(pw)
+  if algorithm in list(hashlib.algorithms_available):
+    h = hashlib.new(algorithm)
+    h.update(pw.encode())
     if hex:
-      enc = enc.hexdigest()
+      enc = h.hexdigest()
     else:
-      enc = enc.digest()
-  else:
-    for id, prefix, scheme in AuthEncoding._schemes:
-      if algorithm.upper() == id:
-        enc = scheme.encrypt(pw)
+      enc = h.digest()
   return enc
 
-
 security.declarePublic('encrypt_ordtype')
 def encrypt_ordtype(s):
   """
@@ -621,8 +612,6 @@ def unencode( p, enc='utf-8'):
       p[key] = unencode(p[key],enc)
   elif isinstance(p, list):
     p = [unencode(x,enc) for x in p]
-  elif six.PY2 and is_str(p):
-    p = pybytes(p,enc)
   return p
 
 
@@ -661,19 +650,6 @@ def id_quote(s, mapping={
   return s
 
 
-security.declarePublic('form_quote')
-def form_quote(text, REQUEST):
-  """
-  Remove <form>-tags for Management Interface.
-  """
-  rtn = text
-  if isManagementInterface(REQUEST):
-    rtn = re.sub( '<form(.*?)>', '<noform\\1>', rtn)
-    rtn = re.sub( ' name="lang"', ' name="_lang"', rtn)
-    rtn = re.sub( '</form(.*?)>', '</noform\\1>', rtn)
-  return rtn
-
-
 def qs_append(qs, p, v):
   """
     Append to query-string.
@@ -704,39 +680,41 @@ def nvl(a1, a2, n=None):
     return a2
 
 
-class SessionBTreeWrapper:
-  
-  def __init__(self, t):
-    self.t = t
-  
-  security.declarePublic('set')
-  def set(self, k, v):
-    self.t.update({k:v})
-    return v
-
-  security.declarePublic('get')
-  def get(self, k, v=None):
-    return self.t.get(k,v)
-
-
 security.declarePublic('get_session')
 def get_session(context):
   """
   Get http-session.
   """
-  req = getattr( context, 'REQUEST', None)
-  req_session = req.get('SESSION',req.environ.get('beaker.session',None))
-  session = None
-  if req_session:
-    if req_session.get('__zms_session__') is None:
-      from BTrees.OOBTree import OOBTree
-      try:
-        req_session.set('__zms_session__',OOBTree()) # FIXME: why does set work?
-      except:
-        req_session['__zms_session__'] = OOBTree()
-    session = SessionBTreeWrapper(req_session.get('__zms_session__'))
+  request = getattr( context, 'REQUEST', None)
+  if request.get('SESSION', None) == None:
+    create_session_storage_if_neccessary(context)
+  session = request.get('SESSION',request.environ.get('beaker.session',None))
   return session
 
+security.declarePublic('create_session_storage_if_neccessary')
+def create_session_storage_if_neccessary(context):
+  """
+  Ensure containers for temporary data.
+  """
+  from OFS.Folder import Folder
+  from Products.Transience.Transience import TransientObjectContainer
+
+  root = context.getPhysicalRoot()
+  if not 'temp_folder' in root:
+    # Adding a 'folder' is a just fallback
+    # if a 'mount_point' is not available 
+    # like usually configured via zope.conf
+    temp_folder = Folder('temp_folder')
+    root._setObject('temp_folder', temp_folder)
+    # writeLog( context, 'Missing temp_folder added')
+  if not 'session_data' in root.temp_folder:
+    container = TransientObjectContainer(
+        'session_data',
+        title='Session Data Container',
+        timeout_mins=20
+    )
+    root.temp_folder._setObject('session_data', container)
+    # writeLog( context, 'Missing session_data-container added')
 
 security.declarePublic('get_session_value')
 def get_session_value(context, key, defaultValue=None):
@@ -862,7 +840,8 @@ def http_import(context, url, method='GET', auth=None, parse_qs=0, timeout=10, h
   @rtype: C{str}
   """
   # Parse URL.
-  u = urllib_urlparse(url)
+  import urllib.parse
+  u = urllib.parse.urlparse(url)
   writeLog( context, "[http_import.%s]: %s"%(method, str(u)))
   scheme = u[0]
   netloc = u[1]
@@ -884,12 +863,12 @@ def http_import(context, url, method='GET', auth=None, parse_qs=0, timeout=10, h
       netloc = proxy
 
   # Open HTTP connection.
-  from six.moves import http_client
+  import http.client
   writeLog( context, "[http_import.%s]: %sConnection(%s) -> %s"%(method, scheme, netloc, path))
   if scheme == 'http':
-    conn = http_client.HTTPConnection(netloc, timeout=timeout)
+    conn = http.client.HTTPConnection(netloc, timeout=timeout)
   else:
-    conn = http_client.HTTPSConnection(netloc, timeout=timeout)
+    conn = http.client.HTTPSConnection(netloc, timeout=timeout)
 
   # Set request-headers.
   if auth is not None:
@@ -1358,6 +1337,24 @@ def parseLangFmtDate(s):
 #
 ############################################################################
 
+security.declarePublic('operator_contains')
+def operator_contains(c, v, ignorecase=False):
+  """
+  Check if collection contains value.
+  @param c: Collection
+  @type c: C{list|set|tuple}
+  @param v: Value
+  @type v: C{any}
+  @param ignorecase: Ignore Case-Sensitivity
+  @type ignorecase: C{Bool}
+  @return: Collection contains value
+  @rtype: C{Bool}
+  """
+  if ignorecase:
+    return v.lower() in [x.lower() for x in c]
+  else:
+    return v in c
+
 security.declarePublic('operator_gettype')
 def operator_gettype(v):
   """
@@ -1396,11 +1393,11 @@ def operator_getitem(a, b, c=None, ignorecase=True):
   @type b: C{any}
   @param c: Default-Value
   @type c: C{any}
+  @param ignorecase: Ignore Case-Sensitivity
+  @type ignorecase: C{Bool}
   @rtype: C{any}
   """
-  if ignorecase and is_str(b):
-    b = pybytes(b)
-  if ignorecase and is_bytes(b):
+  if ignorecase and ( isinstance(b, bytes) or isinstance(b, str) ):
     flags = re.IGNORECASE
     pattern = '^%s$'%b
     for key in a:
@@ -1686,9 +1683,8 @@ def string_list(s, sep='\n', trim=True):
       l.append(i)
   return l
 
-if six.PY3:
-  def cmp(x, y):
-      return (x > y) - (x < y)
+def cmp(x, y):
+  return (x > y) - (x < y)
 
 security.declarePublic('is_equal')
 def is_equal(x, y):
@@ -1713,6 +1709,13 @@ def is_equal(x, y):
       return cmp(x.toXml(),y.toXml())==0
   return cmp(x, y)==0
 
+security.declarePublic('parse_json')
+def parse_json(*args, **kwargs):
+  """
+  Returns an object representation of the json-string.
+  @rtype: C{dict|list|int|etc.}
+  """
+  return json.loads(*args, **kwargs)
 
 security.declarePublic('str_json')
 def str_json(i, encoding='ascii', errors='xmlcharrefreplace', formatted=False, level=0, allow_booleans=True, sort_keys=True):
@@ -1732,37 +1735,36 @@ def str_json(i, encoding='ascii', errors='xmlcharrefreplace', formatted=False, l
         + (['','\n'][formatted]+(['','\t'][formatted]*level)+',').join(['"%s":%s'%(x,str_json(i[x],encoding,errors,formatted,level+1,allow_booleans,sort_keys)) for x in k]) \
         + '}'
   elif type(i) is time.struct_time:
-    try:
-      return '"%s"'%format_datetime_iso(i)
-    except:
-      pass
-  elif type(i) is int or type(i) is float or type(i) is bool:
+    return '"%s"'%format_datetime_iso(i)
+  elif type(i) is int or type(i) is float:
     return json.dumps(i)
+  elif type(i) is bool:
+    if allow_booleans:
+      return json.dumps(i)
+    else:
+      return str(i)
   elif i is not None:
     if allow_booleans and i in ['true','false']:
       return i
     else:
       if not is_str(i):
-        i = pystr(i)
+        i = str(i)
     return '"%s"'%(i.replace('\\','\\\\').replace('"','\\"').replace('\n','\\n').replace('\r','\\r'))
   return '""'
 
 
 security.declarePublic('str_item')
-def str_item(i):
+def str_item(i, f=False):
   """
   Returns a string representation of the item.
   @rtype: C{str}
   """
-  if isinstance(i, list) or isinstance(i, tuple):
-    return '\n'.join([str_item(x) for x in i])
+  if isinstance(i, time.struct_time):
+    return format_datetime_iso(i)
+  elif isinstance(i, list) or isinstance(i, tuple):
+    return '\n'.join([str_item(x,f) for x in i])
   elif isinstance(i, dict):
-    return '\n'.join([str_item(i[x]) for x in i])
-  elif isinstance(i, time.struct_time):
-    try:
-      return format_datetime_iso(i)
-    except:
-      pass
+    return '\n'.join([str_item(i[x],f) for x in i if not f or not x.startswith('_')])
   if i is not None:
     return str(i)
   return ''
@@ -1969,16 +1971,16 @@ def parseXmlString(xml):
   """
   Parse value from ZMS XML-Structure.
   @param xml
-  @type xml: C{str} or C{BytesIO}
+  @type xml: C{str} or C{io.BytesIO}
   @return: C{list} or C{dict}
   @rtype: C{any}
   """
   from Products.zms import _xmllib
   builder = _xmllib.XmlAttrBuilder()
-  if is_str(xml):
-    xml = pybytes(xml,'utf-8')
-  if is_bytes(xml):
-    xml = PyBytesIO(xml)
+  if isinstance(xml,str):
+    xml = bytes(xml,'utf-8')
+  if isinstance(xml,bytes):
+    xml = io.BytesIO(xml)
   v = builder.parse(xml)
   return v
 
@@ -1992,7 +1994,7 @@ def processData(context, processId, data, trans=None):
   @param processId: the process-id
   @type processId: C{str}
   @param data: the xml-data
-  @type data: C{str} or C{BytesIO}
+  @type data: C{str} or C{io.BytesIO}
   @param trans: the transformation
   @type trans: C{str}
   @return: the transformed data
@@ -2019,7 +2021,7 @@ def dt_executable(context, v):
   @return:
   @rtype: C{Bool}
   """
-  if _globals.is_str_type(v):
+  if isinstance(v, bytes) or isinstance(v, str):
     if v.startswith('##'):
       return 'py'
     elif v.find('<tal:') >= 0:
@@ -2078,8 +2080,6 @@ def dt_html(context, value, REQUEST):
   value = re.sub( '</dtml-var>', '', value)
   dtml = DocumentTemplate.DT_HTML.HTML(value)
   value = dtml( context, REQUEST)
-  if type(value) is bytes:
-    value = value.decode('utf-8','ignore')
   return value
 
 def dt_py( context, script, kw={}):
@@ -2141,8 +2141,6 @@ def dt_tal(context, text, options={}):
   pt.setEnv(context, options)
   request = context.REQUEST
   rendered = pt.pt_render(extra_context={'here':context,'request':request})
-  if isinstance(rendered,bytes):
-    rendered = rendered.decode()
   return rendered
 
 #}
@@ -2251,10 +2249,6 @@ def sendMail(context, mto, msubject, mbody, REQUEST=None, mattach=None):
         part.add_header('Content-Disposition', 'attachment; filename="%s"'%filename)
         mime_msg.attach(part)
 
-      # TODO: Handle data from filesystem or other sources
-      elif isinstance(filedata, file):
-        raise NotImplementedError
-
   # Send mail.
   try:
     #writeBlock( context, "[sendMail]: %s"%mime_msg.as_string())
@@ -2265,15 +2259,6 @@ def sendMail(context, mto, msubject, mbody, REQUEST=None, mattach=None):
     return -1
 
 
-security.declarePublic('extutil')
-def extutil():
-  """
-  Returns util to handle zms3.extensions
-  """
-  from Products.zms import _extutil
-  return _extutil.ZMSExtensions()
-
-
 security.declarePublic('getPlugin')
 def getPlugin( context, path, options={}):
   """
@@ -2310,10 +2295,10 @@ def getTempFile( context, id):
   temp_file = getattr(temp_folder,id)
   data = temp_file.data
   b = data
-  if is_str(data):
-    b = pybytes(data)
-  elif not is_bytes(data):
-    b = pybytes(b'')
+  if isinstance(data,str):
+    b = bytes(data)
+  elif not isinstance(data,bytes):
+    b = bytes(b'')
     while data is not None:
        b += data.data
        data=data.next
@@ -2353,4 +2338,4 @@ class initutil(object):
 
 security.apply(globals())
 
-################################################################################
\ No newline at end of file
+################################################################################
