@@ -95,33 +95,12 @@ zope.event.subscribers.append(subscriber)
 # ------------------------------------------------------------------------------
 #  importTheme:
 # ------------------------------------------------------------------------------
-def importTheme(folder, theme):
-  
+def importTheme(self, theme):
   filename = _fileutil.extractFilename(theme)
-  id = filename[:filename.rfind('.')]
-  
-  if filename.endswith('.zexp'):
-    ### Store copy of ZEXP in INSTANCE_HOME/import-folder.
-    filepath = standard.getINSTANCE_HOME() + '/import/' + filename
-    if theme.startswith('http://'):
-      initutil = standard.initutil()
-      initutil.setConfProperty('HTTP.proxy',REQUEST.get('http_proxy',''))
-      zexp = standard.http_import( initutil, theme)
-      _fileutil.exportObj( zexp, filepath)
-    else:
-      packagepath = package_home(globals()) + '/import/' + filename
-      try:
-        os.stat(_fileutil.getOSPath(filepath))
-      except OSError:
-        shutil.copy( packagepath, filepath)
-    
-    ### Import theme from ZEXP.
-    _fileutil.importZexp( folder, filename)
-  
-  else:
-    id = filename[:filename.find('-')]
-    _confmanager.initConf(folder.content, id, remote=False)
-  
+  id = filename[:filename.rfind('-')]
+  filepath = package_home(globals()) + '/import/'
+  path = filepath + filename
+  self.importConf(path)
   return id
 
 
@@ -160,14 +139,15 @@ def initZMS(self, id, titlealt, title, lang, manage_lang, REQUEST):
   obj.setConfProperty('HTTP.proxy', REQUEST.get('http_proxy', ''))
   obj.setConfProperty('ZMS.autocommit', 1)
 
-  ### Init ZMS object-model.
-  _confmanager.initConf(obj, 'com.zms.foundation', remote=False)
-  _confmanager.initConf(obj, 'com.zms.foundation.bootstrap', remote=False)
-  _confmanager.initConf(obj, 'com.zms.foundation.theme', remote=False)
-  _confmanager.initConf(obj, 'com.zms.index', remote=False)
+  ### Init ZMS default content-model.
+  _confmanager.initConf(obj, 'conf:com.zms.foundation*')
+  _confmanager.initConf(obj, 'conf:com.zms.index')
+
+  ### Init ZMS default actions.
+  _confmanager.initConf(obj, 'conf:manage_tab_*')
 
   ### Init default-configuration.
-  _confmanager.initConf(obj, 'default', remote=False)
+  _confmanager.initConf(obj, ':default')
 
   ### Init Role-Definitions and Permission Settings.
   obj.initRoleDefs()
@@ -219,7 +199,7 @@ def manage_addZMS(self, lang, manage_lang, REQUEST, RESPONSE):
     obj = initZMS(homeElmnt, 'content', titlealt, title, lang, manage_lang, REQUEST)
     
     ##### Add Theme ####
-    themeId = importTheme(homeElmnt,REQUEST['theme'])
+    themeId = importTheme(obj,REQUEST['theme'])
     obj.setConfProperty('ZMS.theme',themeId)
 
     ##### Default content ####
@@ -275,6 +255,10 @@ class ZMS(
 
     # Management Permissions.
     # -----------------------
+    __viewPermissions__ = (
+        'manage', 'manage_main', 'manage_container', 'manage_workspace', 'manage_menu',
+        'manage_ajaxGetChildNodes',
+        )
     __administratorPermissions__ = (
         'manage_customize',
         'manage_customizeInstalledProducts',
@@ -283,7 +267,6 @@ class ZMS(
         'manage_customizeDesign', 'manage_customizeDesignForm',
         )
     __authorPermissions__ = (
-        'manage', 'manage_main', 'manage_main_iframe', 'manage_workspace',
         'manage_addZMSModule',
         'manage_deleteObjs', 'manage_undoObjs',
         'manage_moveObjUp', 'manage_moveObjDown', 'manage_moveObjToPos',
@@ -291,6 +274,7 @@ class ZMS(
         'manage_ajaxDragDrop', 'manage_ajaxZMIActions',
         'manage_properties', 'manage_changeProperties', 'manage_changeTempBlobjProperty',
         'manage_wfTransition', 'manage_wfTransitionFinalize',
+        'manage_RefForm',
         'manage_userForm', 'manage_user',
         'manage_importexport', 'manage_import', 'manage_export',
         'manage_executeMetacmd',
@@ -299,6 +283,7 @@ class ZMS(
         'manage_users', 'manage_users_sitemap', 'manage_userProperties', 'manage_roleProperties', 'userdefined_roles',
         )
     __ac_permissions__=(
+        ('View', __viewPermissions__),
         ('ZMS Administrator', __administratorPermissions__),
         ('ZMS Author', __authorPermissions__),
         ('ZMS UserAdministrator', __userAdministratorPermissions__),
@@ -378,63 +363,13 @@ class ZMS(
     #  Get version.
     # --------------------------------------------------------------------------
     def zms_version(self):
-      """
-      Try to obtain the version from package info if installed by pip
-      otherwise read from version.txt and from svnversion
-        
-        1. installed by pip from PyPI
-          (revision 'XXXX' in version.txt - package info of official releases does not contain revision)
-        2. installed by pip from ZMSLabs w/ info by svn
-          (revision 'svn-rXXXX' in package info due to setup.cfg)
-        3. installed by pip from TAR-Ball w/o info by svn
-          (revision 'XXXX' in version.txt due to nightly build - 'svn0' in package info due to setup.cfg)
-        4. deployed to instance/Products
-          (revision 'XXXX' in version.txt, maybe installed but unused package)
-        5. checked out to instance/Products as working copy from svn 
-          (revision 'REV' in version.txt, maybe installed but unused package)
-      """
-      from pkg_resources import WorkingSet, Requirement
-      zms = WorkingSet().find(Requirement.parse('ZMS3'))
-      pth = _fileutil.getOSPath(package_home(globals()))
-      
-      file = open(_fileutil.getOSPath(pth+'/version.txt'), 'r')
-      version_txt = file.read()
-      version     = version_txt.strip().split('.')
+      file = open(_fileutil.getOSPath(package_home(globals())+'/version.txt'),'r')
+      rtn = file.read()
       file.close()
-      
-      # return plain version info if it is a deployment specific format like 
-      # ZMS3-3.Y.Z.XXXX.prj-ABCD
-      if len(version)!=4:
-        return version_txt
-      
-      # obtain revision and return formatted version info
-      # ZMS3-3.Y.Z.XXXX
-      else:
-        revision = version.pop()
-        # get revision from pip if running as package
-        if (zms is not None) and ('site-packages' in pth):
-          version_pip = str(zms.version)
-          if ('.svn-r' in version_pip):
-            # leave -r in revision to recognize as snapshot below
-            revision = version_pip.strip().split('.svn', 1)[1]        
-        # get revision from svnversion
-        elif (revision == 'REV'):
-          import subprocess
-          version_svn = subprocess.Popen("svnversion",
-            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-            shell=True, cwd=pth, universal_newlines=True)
-          revision = version_svn.communicate()[0].strip()
-        # format revision info
-        if (revision.startswith('export')):
-          revision = ' (unknown build/snapshot)'
-        # at moment there are 4-digits for builds 
-        elif len(revision)==4:
-          revision = ' (build #%s)'%revision
-        # otherwise it is a development snapshot
-        else:
-          revision = ' (snapshot #%s)'%revision.replace('-r', '')
-      
-      return '.'.join(version)+revision
+      zms_custom_version = os.environ.get('ZMS_CUSTOM_VERSION', '')
+      if zms_custom_version != '':
+          rtn += ' ({})'.format(zms_custom_version)
+      return rtn
 
     # --------------------------------------------------------------------------
     #  ZMS.getDocumentElement
